#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from contextlib import contextmanager

from flask_sqlalchemy import SQLAlchemy as _SQLAlchemy, BaseQuery

from .http_code import NotFoundException


class SQLAlchemy(_SQLAlchemy):
    @contextmanager
    def auto_commit(self):
        try:
            yield
            self.session.commit()
        except Exception as e:
            db.session.rollback()
            raise e


class Query(BaseQuery):
    def filter_by(self, **kwargs):
        return super(Query, self).filter_by(**kwargs)

    def get_or_404(self, ident, description=None):
        rv = self.get(ident)
        if not rv:
            raise NotFoundException()
        return rv

    def first_or_404(self, description=None):
        rv = self.first()
        if not rv:
            if description:
                raise NotFoundException(data=description)
            raise NotFoundException()
        return rv

    def paginate(self, page: int = None, per_page: int = None, **kwargs):
        """"""
        total = self.order_by(None).count()

        if page is None:
            page = 1

        if page < 1:
            page = 1

        if per_page < 1:
            per_page = 10

        if (total / per_page) == int(total / per_page):
            page_total = int(total / per_page)
        else:
            page_total = int(total / per_page) + 1

        if page == 1:
            page_num = 0
        else:
            page_num = per_page * page - per_page

        items = self.limit(per_page).offset(page_num).all()

        return dict(total=total, page_total=page_total, items=items, page=page, pagesize=len(items))


db = SQLAlchemy(query_class=Query)


class Base(db.Model):
    __abstract__ = True

    def __init__(self):
        pass

    def __getitem__(self, item):
        return getattr(self, item)

    def set_attrs(self, attrs_dict):
        for key, value in attrs_dict.items():
            if hasattr(self, key) and key != 'id':
                setattr(self, key, value)

    def keys(self):
        return self.fields

    def hide(self, *keys):
        for key in keys:
            self.fields.remove(key)
        return self

    def append(self, *keys):
        for key in keys:
            self.fields.append(key)
        return self



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21


class ExceptionBase(Exception):
    """Basic abnormality"""
    pass


class ExceptionService(ExceptionBase):
    """Service exception"""
    pass


class ExceptionServer(ExceptionBase):
    """Service exception"""
    pass


class ExceptionError(ExceptionBase):
    """Error exception"""
    pass


class ExceptionWarning(ExceptionBase):
    """Warning exception"""
    pass


class ExceptionInfo(ExceptionBase):
    """Message exception"""
    pass



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-22
import typing as t
from flask.wrappers import Request as _Request


class Request(_Request):

    def get_json(
            self, force: bool = False, silent: bool = False, cache: bool = True
    ) -> t.Optional[t.Any]:
        """Parse :attr:`data` as JSON.

                If the mimetype does not indicate JSON
                (:mimetype:`application/json`, see :attr:`is_json`), or parsing
                fails, :meth:`on_json_loading_failed` is called and
                its return value is used as the return value. By default this
                raises a 400 Bad Request error.

                :param force: Ignore the mimetype and always try to parse JSON.
                :param silent: Silence mimetype and parsing errors, and
                    return ``None`` instead.
                :param cache: Store the parsed JSON to return for subsequent
                    calls.

                .. versionchanged:: 2.1
                    Raise a 400 error if the content type is incorrect.
                """
        if self.is_json is False:
            return dict()

        if cache and self._cached_json[silent] is not Ellipsis:
            return self._cached_json[silent]

        if not (force or self.is_json):
            if not silent:
                return self.on_json_loading_failed(None)
            else:
                return None

        data = self.get_data(cache=cache)

        try:
            rv = self.json_module.loads(data)
        except ValueError as e:
            if silent:
                rv = None

                if cache:
                    normal_rv, _ = self._cached_json
                    self._cached_json = (normal_rv, rv)
            else:
                rv = self.on_json_loading_failed(e)

                if cache:
                    _, silent_rv = self._cached_json
                    self._cached_json = (rv, silent_rv)
        else:
            if cache:
                self._cached_json = (rv, rv)

        return rv



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
import json
import os.path
import time

from fourcats_utils import logger
from flask import g, request, Response

from ..refactor.api import Api


class GlobalLogger:
    """
    Print custom request log after request.
    """

    def __init__(self, response: Response, api: Api, level: str = "info"):
        """"""
        try:
            if "swagger" not in request.path:
                self.__logger(response=response, api=api, level=level)
        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.error(e)

    def __logger(self, response: Response, api: Api, level: str = "info"):
        """
        Print logs.
        :return:
        """

        method = request.method
        path = request.path
        swagger_dict = api.__schema__

        if request.url_rule is not None:
            rule_basename = os.path.basename(request.url_rule.rule)
            if ":" in rule_basename:
                basename = "{" + rule_basename.split(":")[1].replace(">", "") + "}"
            else:
                basename = rule_basename.replace("<", "{").replace(">", "}")
            rule = request.url_rule.rule.replace(rule_basename, basename)
        else:
            rule = ""

        describe = swagger_dict.get("paths").get(rule, dict()).get(method.lower(), dict()).get("summary")
        params = self.__get_params(_request=request)
        uip = request.remote_addr
        http_code = response.status_code

        log_msg = {
            "describe": describe, "method": method, "path": path, "request": params, "uip": uip, "http_code": http_code,
            "response": json.loads(bytes.decode(response.data)) if response.data else list(),
            "request_time": g.request_time, "response_time": int(time.time()), "use_time": time.time() - g.request_time
        }

        getattr(logger, level)(json.dumps(log_msg, ensure_ascii=False))

    def __get_params(self, _request):
        """
        Get all parameters.
        :return:
        """
        parameter = dict()
        parameter["data"] = self.__change(_request.data)
        parameter["args"] = self.__change(_request.args)
        parameter["form"] = self.__change(_request.form)
        if _request.is_json is True:
            parameter["json"] = self.__change(_request.json)
        else:
            parameter["json"] = dict()
        parameter["values"] = self.__change(_request.values)
        return parameter

    @staticmethod
    def __change(data):
        """
        Detect the parameter type.
        :param data: parameter
        :return:
        """
        result = data
        if isinstance(data, bytes):
            result = bytes.decode(data)
            if result:
                try:
                    result = json.loads(result)
                except json.decoder.JSONDecodeError as e:
                    logger.error(f"{e} - {result}")
            else:
                result = None
        return dict() if result is None else result



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
import typing

if typing.TYPE_CHECKING:
    from _typeshed.wsgi import WSGIEnvironment

from flask import json, g
from werkzeug.exceptions import HTTPException


class ApiResponse(HTTPException):
    code = 500
    message = "Sorry, an unknown error occurred in the system. (*￣︶￣)!"
    state_code = 0

    def __init__(self, http_code=None, message=None, state_code=None, data=None, paging=False):
        if http_code is not None:
            self.code = http_code

        if message is not None:
            self.message = message

        if state_code is not None:
            self.state_code = state_code

        self.paging = paging
        self.data = data if data is not None else list()
        super(ApiResponse, self).__init__(message, None)

    def get_body(
            self,
            environ: typing.Optional["WSGIEnvironment"] = None,
            scope: typing.Optional[dict] = None,
    ) -> str:
        body = dict(message=self.message, code=self.state_code, timestamp=g.request_time, data=self.data)
        if self.paging:
            body["data"] = self.get_page(**self.data)
        return json.dumps(body)

    def get_headers(
            self,
            environ: typing.Optional["WSGIEnvironment"] = None,
            scope: typing.Optional[dict] = None,
    ) -> typing.List[typing.Tuple[str, str]]:
        return [("Content-Type", "application/json; charset=utf-8")]

    @staticmethod
    def get_page(total, page_total, page, pagesize, items):
        body = dict(total=total, page_total=page_total, page=page, pagesize=pagesize, items=items)
        return body



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from flask import request
from flask_restx.reqparse import RequestParser as _RequestParser

from .http_code import ParameterException


class RequestParser(_RequestParser):
    """
    Custom parameter parser.
    """

    def parse_args(self, req=None, strict=False):
        """
        Parse all arguments from the provided request and return the results as a ParseResult
        :rtype: ParseResult
        :param req:
        :param strict: if req includes args not in parser, throw 400 BadRequest exception
        :return: the parsed results as :class:`ParseResult` (or any class defined as :attr:`result_class`)
        """
        if req is None:
            req = request

        result = self.result_class()

        # A record of arguments not yet parsed; as each is found
        # among self.args, it will be popped out
        req.unparsed_arguments = dict(self.argument_class('').source(req)) if strict else {}
        errors = {}
        for arg in self.args:
            value, found = arg.parse(req, self.bundle_errors)
            if isinstance(value, ValueError):
                errors.update(found)
                found = None
            if found or arg.store_missing:
                result[arg.dest or arg.name] = value
        if errors:
            raise ParameterException(data=errors)

        if strict and req.unparsed_arguments:
            arguments = ', '.join(req.unparsed_arguments.keys())
            msg = 'Unknown arguments: {0}'.format(arguments)
            raise ParameterException(data=msg)

        return result
